
#pragma OPENCL EXTENSION cl_khr_fp64 : enable
__kernel void prefixSum(__global int *input,
						__global int *temp,
						__global int *output,
						__global int *sums,
						const unsigned int inputSize,
						const unsigned int height,
						const unsigned int workSize)
{
    // Get our thread IDs
    int globalId = get_global_id(0);
	int localId = get_local_id(0);

	// Copy input to the temporary array (which represents a tree)
	temp[globalId * 2] = input[globalId * 2];
	temp[globalId * 2 + 1] = input[globalId * 2 + 1];

	// Perform the up-sweep phase
	for (int n = 1; n < height; n++) {
		// All threads ned to be done with the last level of the tree before we go to the next one
		barrier(CLK_GLOBAL_MEM_FENCE);
		// Add the left and right subnode to the current node assigned to this thread
		temp[n*inputSize + globalId*2 - localId] = temp[(n - 1)*inputSize + globalId * 2] + temp[(n - 1)*inputSize + globalId * 2 + 1];
	}
	// Copy highest value to the sums array
	// No barrier required, same thread places that value there.
	if (localId == 0) {
		sums[globalId / (workSize/2)] = temp[(height - 1)*inputSize + globalId*2];
	}
	barrier(CLK_GLOBAL_MEM_FENCE);

	// Perform the down-sweep phase
	// Set root to 0
	temp[(height - 1)*inputSize + globalId*2] = 0;
	for (int n = height - 1; n > 0; n--) {
		// All threads ned to be done with the last level of the tree before we go to the next one
		barrier(CLK_GLOBAL_MEM_FENCE);
		temp[(n - 1)*inputSize + globalId * 2 + 1] = temp[(n - 1)*inputSize + globalId * 2] + temp[n*inputSize + globalId * 2 - localId];
		temp[(n - 1)*inputSize + globalId * 2] = temp[n*inputSize + globalId * 2 - localId];
		// right:temp[(n - 1)*inputSize + globalId * 2 + 1]
		// left: temp[(n - 1)*inputSize + globalId * 2]
		// root: temp[n*inputSize + globalId*2 - localId]
	}
	// Copy the leafs of the tree (located in the temp array) to the output array
	barrier(CLK_GLOBAL_MEM_FENCE);
	output[globalId * 2] = temp[globalId * 2];
	output[globalId * 2 + 1] = temp[globalId * 2 + 1];
}


__kernel void adder(__global int *data,
					__global int *sums,
					const unsigned int workSize)
{
	// Get our thread IDs
	int globalId = get_global_id(0);
	int localId = get_local_id(0);

	data[globalId * 2] = data[globalId * 2] + sums[globalId / (workSize / 2)];
	data[globalId * 2 +1] = data[globalId * 2 +1] + sums[(globalId*2+1) / workSize];
}


