
#pragma OPENCL EXTENSION cl_khr_fp64 : enable
__kernel void prefixSum(__global int *input,
						__global int *output,
						__global int *temp,
						const unsigned int inputSize)
{
    // Get our global thread ID
    int id = get_global_id(0);

	// Copy input to the temporary array (which represents a tree)
	temp[id] = 0;
	temp[id + inputSize] = input[id];

	// Perform the up-sweep phase
	for (int n = inputSize>>1; n > 0; n >>=1) {
		// All threads ned to be done with the last level of the tree before we go to the next one
		barrier(CLK_GLOBAL_MEM_FENCE);
		if (id < n) { // TODO: Test if this decreases performance
			// Add the left and right subnode to the current node assigned to this thread
			temp[n+id] = temp[(n+id)*2] + temp[(n+id)*2+1];
		}
		
	}

	// Perform the down-sweep phase
	// Set root to 0 with thread 0
	if (id == 0) {
		temp[1] = 0;
	}
	int n;
	for (n = 1; n < (inputSize); n <<= 1) {
		// All threads ned to be done with the last level of the tree before we go to the next one
		barrier(CLK_GLOBAL_MEM_FENCE);
		if (id < n) {
			temp[(n + id) * 2 + 1] = temp[n + id] + temp[(n + id) * 2];	// Set the value of the right child
			temp[(n + id) * 2] = temp[n + id];							// Set the value of the left child
		}
	}
	// Copy the leafs of the tree (located in the temp array) to the output array
	barrier(CLK_GLOBAL_MEM_FENCE);
	output[id] = temp[id + inputSize];
}


