#pragma
#include "Utils.h"
bool printDebug = false;
bool hardwareFound = false;
sclHard hardware;

void debug(char* fmt, ...) {
	if (printDebug) {
		va_list args;
		va_start(args, fmt);
		vprintf(fmt, args);
		va_end(args);
	}
}

int* calculateCPU(int inputSize, int* input) {
	int* output = new int[inputSize]();
	if (inputSize > 0) {
		output[0] = input[0];
	}
	for (int i = 1; i < inputSize; i++) {
		output[i] = output[i - 1] + input[i - 1];
	}
	return output;
}

sclHard getHardware() {
	if (!hardwareFound) {
		hardwareFound = true;
		int found;
		hardware = sclGetGPUHardware(0, &found);
	}
	return hardware;
}


/* INPUT FUNCTIONS */
int* generateIncreasingInput(int size) {
	int *result = new int[size];
	for (int i = 0; i < size; i++)
	{
		result[i] = i;
	}
	return result;
}

int* padInputTo(int* input, int size, int paddedSize) {
	int *result = new int[paddedSize];
	for (int i = 0; i < paddedSize; i++)
	{
		if (i < size) {
			result[i] = input[i];
		}
		else {
			result[i] = 0;
		}
	}
	return result;
}


/* PRINT FUNCTIONS */
void printArray(int size, int* input) {
	if (size >= 1) {
		debug("%d", input[0]);
	}
	for (int i = 1; i < size; i++) {
		debug(", %d", input[i]);
	}
	debug("\n");
}

void printArrays(int size, int* input, int* one, int* two) {
	debug("%-5s %-11s %-11s %-11s\n", "#", "Input:", "CPU:", "GPU:");
	for (int i = 0; i < size; i++) {
		debug("%-5d %-11d %-11d %-11d\n", i, input[i], one[i], two[i]);
	}
}

void printArraysDepth(int depth, int size, int* input, int* one, int* two) {
	debug("%d: %-5s %-11s %-11s %-11s\n", depth, "#", "Input:", "CPU:", "GPU:");
	for (int i = 0; i < size; i++) {
		debug("%d: %-5d %-11d %-11d %-11d\n", depth, i, input[i], one[i], two[i]);
	}
}

void printArrayTree(int size, int* array) {
	debug("%-5s %s", "#", "Temporary array values:");
	int lastLog = -1;
	for (int i = 1; i < size * 2; i++) {
		int log = log2(i);
		if (log != lastLog) {
			debug("\n%-5d ", log);
			lastLog = log;
		}
		debug("%-5d  ", array[i]);
	}
}

void printMultiArray(int height, int width, int* array) {
	debug("Temporary array values:\n");
	for (int i = 0; i < width*height; i++) {
		debug("%-11d ", array[i]);
		if ((i+1)%height == 0) {
			debug("\n");
		}
	}
}

void printMultiArrayDepth(int depth, int height, int width, int* array) {
	debug("%d: Temporary array values:\n", depth);
	for (int i = 0; i < width*height; i++) {
		if (i%height == 0) {
			debug("%d: ", depth);
		}
		debug("%-11d ", array[i]);
		if ((i + 1) % height == 0) {
			debug("\n");
		}
	}
}


/* OPENCL FUNCTIONS */
char* getHardwareName(sclHard hardware) {
	size_t valueSize;
	clGetDeviceInfo(hardware.device, CL_DEVICE_NAME, 0, NULL, &valueSize);
	char* value = new char[valueSize];
	clGetDeviceInfo(hardware.device, CL_DEVICE_NAME, valueSize, value, NULL);
	return value;
}

size_t getMaximumWorkgroupSize(sclHard hardware) {
	size_t maxWorkGroupResult = -1;
	if (clGetDeviceInfo(hardware.device, CL_DEVICE_MAX_WORK_GROUP_SIZE, sizeof(size_t), (void*)&maxWorkGroupResult, NULL) != CL_SUCCESS || maxWorkGroupResult < 1) {
		printf("[WARNING] Could not retrieve maximum workgroup size from device, using default of 128.\n");
		return 128;
	}
	else {
		return maxWorkGroupResult;
	}
}